#ifndef PIANOHPP
#define PIANOHPP

#include "graphics.hpp"
#include "icon.hpp"
#include "box.hpp"
#include "events.hpp"

#define WHITE_KEYS_PER_OCTAVE 7

// there is no MI# note, but count it anyway to
// simplify the computation of pitch
#define BLACK_KEYS_PER_OCTAVE 6

#define KEYS_PER_OCTAVE (WHITE_KEYS_PER_OCTAVE + \
			 BLACK_KEYS_PER_OCTAVE)

// relate all key dimensions to the white key width
// so keys can easily be scaled up or down without
// altering their aspect ratio

#define WHITE_KEY_WIDTH 38
#define WHITE_KEY_HEIGHT (9 * WHITE_KEY_WIDTH / 2)
#define BLACK_KEY_WIDTH (5 * WHITE_KEY_WIDTH / 8)
#define BLACK_KEY_HEIGHT (5 * WHITE_KEY_WIDTH / 2)
#define KEYBOARD_WIDTH \
   ( (WHITE_KEY_WIDTH + 1) * WHITE_KEYS_PER_OCTAVE)

class PianoKey: public Icon {

	friend class PianoKeyboard;
	PianoKeyboard* parent;
	COLOR_TYPE key_color;
	int key_frequency;
	Box* key;

public:

	PianoKey(PianoKeyboard*, int, int,
		 int, int, COLOR_TYPE, int);
	void Display();
	void LeftButtonPressed();
	void LeftButtonReleased();
	~PianoKey();
};

class PianoKeyboard: public Icon {

	PianoKey* key [KEYS_PER_OCTAVE + 1];
	void CreateWhiteKeys(int, int, int);
	void CreateBlackKeys(int, int, int);

public:

	PianoKeyboard(int, int, int octave = 1);
	void Display();
	virtual void KeyPressed(int frequency);
	virtual void KeyReleased(int frequency);
	virtual ~PianoKeyboard();
};

#endif
